@@ -19,12 +19,10 @@ from warnings import warn
 import re
 
 from six import string_types
-from six.moves.urllib.parse import urlparse
+from six.moves.urllib.parse import urlparse, quote_plus
 
 from .core.templates import (
-    AUTOLOAD_JS, AUTOLOAD_NB_JS, AUTOLOAD_TAG,
-    FILE, NOTEBOOK_DIV, PLOT_DIV, DOC_JS, SCRIPT_TAG
-)
+    AUTOLOAD_JS, AUTOLOAD_TAG, DOC_JS, FILE, NOTEBOOK_DIV, PLOT_DIV, SCRIPT_TAG)
 from .core.json_encoder import serialize_json
 from .document import Document, DEFAULT_TITLE
 from .model import Model
@@ -258,37 +256,28 @@ def components(models, wrap_script=True, wrap_plot_info=True, theme=FromCurdoc):
     else:
         return script, tuple(results)
 
-def _use_widgets(objs):
-    from .models.widgets import Widget
-
-    def _needs_widgets(obj):
-        return isinstance(obj, Widget)
-
+def _any(objs, query):
     for obj in objs:
         if isinstance(obj, Document):
-            if _use_widgets(obj.roots):
+            if _any(obj.roots, query):
                 return True
         else:
-            if any(_needs_widgets(ref) for ref in obj.references()):
+            if any(query(ref) for ref in obj.references()):
                 return True
     else:
         return False
 
+def _use_widgets(objs):
+    from .models.widgets import Widget
+    return _any(objs, lambda obj: isinstance(obj, Widget))
+
+def _use_tables(objs):
+    from .models.widgets import TableWidget
+    return _any(objs, lambda obj: isinstance(obj, TableWidget))
+
 def _use_gl(objs):
     from .models.plots import Plot
-
-    def _needs_gl(obj):
-        return isinstance(obj, Plot) and obj.output_backend == "webgl"
-
-    for obj in objs:
-        if isinstance(obj, Document):
-            if _use_gl(obj.roots):
-                return True
-        else:
-            if any(_needs_gl(ref) for ref in obj.references()):
-                return True
-    else:
-        return False
+    return _any(objs, lambda obj: isinstance(obj, Plot) and obj.output_backend == "webgl")
 
 def _bundle_for_objs_and_resources(objs, resources):
     if isinstance(resources, BaseResources):
@@ -308,28 +297,81 @@ def _bundle_for_objs_and_resources(objs, resources):
 
     # XXX: force all components on server and in notebook, because we don't know in advance what will be used
     use_widgets =  _use_widgets(objs) if objs else True
+    use_tables  =  _use_tables(objs)  if objs else True
     use_gl      =  _use_gl(objs)      if objs else True
 
     if js_resources:
         js_resources = deepcopy(js_resources)
-        if not use_widgets and "bokeh-widgets" in js_resources.components:
-            js_resources.components.remove("bokeh-widgets")
-        if use_gl and "bokeh-gl" not in js_resources.components:
-            js_resources.components.append("bokeh-gl")
+        if not use_widgets and "bokeh-widgets" in js_resources.js_components:
+            js_resources.js_components.remove("bokeh-widgets")
+        if not use_tables and "bokeh-tables" in js_resources.js_components:
+            js_resources.js_components.remove("bokeh-tables")
+        if not use_gl and "bokeh-gl" in js_resources.js_components:
+            js_resources.js_components.remove("bokeh-gl")
         bokeh_js = js_resources.render_js()
     else:
         bokeh_js = None
 
     if css_resources:
         css_resources = deepcopy(css_resources)
-        if not use_widgets and "bokeh-widgets" in css_resources.components:
-            css_resources.components.remove("bokeh-widgets")
+        if not use_widgets and "bokeh-widgets" in css_resources.css_components:
+            css_resources.css_components.remove("bokeh-widgets")
+        if not use_tables and "bokeh-tables" in css_resources.css_components:
+            css_resources.css_components.remove("bokeh-tables")
         bokeh_css = css_resources.render_css()
     else:
         bokeh_css = None
 
     return bokeh_js, bokeh_css
 
+def notebook_content(model, notebook_comms_target=None, theme=FromCurdoc):
+    ''' Return script and div that will display a Bokeh plot in a Jupyter/
+    Zeppelin Notebook. notebook_comms_target is only supported in Jupyter for
+    now.
+
+    The data for the plot is stored directly in the returned HTML.
+
+    Args:
+        model (Model) : Bokeh object to render
+        notebook_comms_target (str, optional) :
+            A target name for a Jupyter Comms object that can update
+            the document that is rendered to this notebook div
+        theme (Theme, optional) :
+            Defaults to the ``Theme`` instance in the current document.
+            Setting this to ``None`` uses the default theme or the theme
+            already specified in the document. Any other value must be an
+            instance of the ``Theme`` class.
+
+    Returns:
+        script, div
+
+    .. note::
+        Assumes :func:`~bokeh.util.notebook.load_notebook` or the equivalent
+        has already been executed.
+
+    '''
+
+    model = _check_one_model(model)
+
+    # Append models to one document. Either pre-existing or new and render
+    with _ModelInEmptyDocument(model, apply_theme=theme):
+        (docs_json, render_items) = _standalone_docs_json_and_render_items([model])
+
+    item = render_items[0]
+    if notebook_comms_target:
+        item['notebook_comms_target'] = notebook_comms_target
+    else:
+        notebook_comms_target = ''
+
+    script = DOC_JS.render(
+        docs_json=serialize_json(docs_json),
+        render_items=serialize_json(render_items)
+    )
+
+    div = _div_for_render_item(item)
+
+    return encode_utf8(script), encode_utf8(div)
+
 def notebook_div(model, notebook_comms_target=None, theme=FromCurdoc):
     ''' Return HTML for a div that will display a Bokeh plot in a
     Jupyter/Zeppelin Notebook. notebook_comms_target is only supported
@@ -356,32 +398,7 @@ def notebook_div(model, notebook_comms_target=None, theme=FromCurdoc):
         has already been executed.
 
     '''
-    model = _check_one_model(model)
-
-    # Append models to one document. Either pre-existing or new and render
-    with _ModelInEmptyDocument(model, apply_theme=theme):
-        (docs_json, render_items) = _standalone_docs_json_and_render_items([model])
-
-    item = render_items[0]
-    if notebook_comms_target:
-        item['notebook_comms_target'] = notebook_comms_target
-    else:
-        notebook_comms_target = ''
-
-    script = _wrap_in_onload(DOC_JS.render(
-        docs_json=serialize_json(docs_json),
-        render_items=serialize_json(render_items)
-    ))
-
-    js = AUTOLOAD_NB_JS.render(
-        comms_target = notebook_comms_target,
-        js_urls = [],
-        css_urls = [],
-        js_raw = [script],
-        css_raw = "",
-        elementid = item['elementid']
-    )
-    div = _div_for_render_item(item)
+    (js, div) = notebook_content(model, notebook_comms_target=notebook_comms_target, theme=theme)
 
     html = NOTEBOOK_DIV.render(
         plot_script = js,
@@ -483,11 +500,14 @@ def autoload_static(model, resources, script_path):
 
     return encode_utf8(js), encode_utf8(tag)
 
-def autoload_server(model=None, app_path=None, session_id=None, url="default", relative_urls=False):
-    ''' Return a script tag that embeds content from a Bokeh server session.
 
-    Bokeh apps embedded using ``autoload_server`` will NOT set the browser
-    window title.
+def _connect_session_or_document(model=None, app_path=None, session_id=None, url="default", relative_urls=False, resources="default", arguments=None):
+    ''' Return a script tag that embeds content from a Bokeh server. This is
+    a private method not meant to be called directly. Instead it is meant to be
+    called by other methods that thinly wrap around it for different use cases:
+    autoload_server, server_document and server_session.
+
+    Bokeh apps embedded using these methods will NOT set the browser window title.
 
     .. note::
         Typically you will not want to save or re-use the output of this
@@ -528,9 +548,23 @@ def autoload_server(model=None, app_path=None, session_id=None, url="default", r
             when running the Bokeh behind reverse-proxies under certain
             configurations
 
+        resources (str) : A string specifying what resources need to be loaded
+            along with the document.
+
+            If ``default`` then the default JS/CSS bokeh files will be loaded.
+
+            If None then none of the resource files will be loaded. This is
+            useful if you prefer to serve those resource files via other means
+            (e.g. from a caching server). Be careful, however, that the resource
+            files you'll load separately are of the same version as that of the
+            server's, otherwise the rendering may not work correctly.
+
+        arguments (dict[str, str], optional) : A dictionary of the arguments to
+            be passed to Bokeh (default: None)
+
+
     Returns:
-        A ``<script>`` tag that will execute an autoload script loaded
-        from the Bokeh Server.
+        A ``<script>`` tag that will embed content from a Bokeh Server.
 
     Examples:
 
@@ -598,7 +632,7 @@ def autoload_server(model=None, app_path=None, session_id=None, url="default", r
         model_id = model._id
 
     if model_id and session_id is None:
-        raise ValueError("A specific model was passed to autoload_server() but no session_id; "
+        raise ValueError("A specific model was passed to _connect_session_or_document() but no session_id; "
                          "this doesn't work because the server will generate a fresh session "
                          "which won't have the model in it.")
 
@@ -620,6 +654,16 @@ def autoload_server(model=None, app_path=None, session_id=None, url="default", r
     if coords.session_id_allowing_none is not None:
         src_path = src_path + "&bokeh-session-id=" + session_id
 
+    if resources not in ("default", None):
+        raise ValueError("`resources` must be either 'default' or None.")
+    if resources is None:
+        src_path = src_path + "&resources=none"
+
+    if arguments is not None:
+        for key, value in arguments.items():
+            if not key.startswith("bokeh-"):
+                src_path = src_path + "&{}={}".format(quote_plus(str(key)), quote_plus(str(value)))
+
     tag = AUTOLOAD_TAG.render(
         src_path = src_path,
         app_path = app_path,
@@ -629,7 +673,35 @@ def autoload_server(model=None, app_path=None, session_id=None, url="default", r
 
     return encode_utf8(tag)
 
-autoload_server.__doc__ = format_docstring(autoload_server.__doc__, DEFAULT_SERVER_HTTP_URL=DEFAULT_SERVER_HTTP_URL)
+_connect_session_or_document.__doc__ = format_docstring(_connect_session_or_document.__doc__, DEFAULT_SERVER_HTTP_URL=DEFAULT_SERVER_HTTP_URL)
+
+def autoload_server(model=None, app_path=None, session_id=None, url="default", relative_urls=False, arguments=None):
+    ''' Return a script tag that embeds content from a Bokeh server and loads
+    all the necessary JS/CSS resource files. Also accepts an optional dictionary
+    of arguments to be passed to Bokeh.
+    '''
+    deprecated((0, 12, 7), 'bokeh.embed.autoload_server', 'bokeh.embed.server_document or bokeh.embed.server_session')
+    return _connect_session_or_document(model=model, app_path=app_path, session_id=session_id, url=url, relative_urls=relative_urls,
+                                        resources="default", arguments=arguments)
+
+def server_document(url="default", relative_urls=False, resources="default", arguments=None):
+    ''' Works similarly to ``autoload_server`` except a new session will be
+    systematically generated and an entire document will be returned. Also
+    accepts an optional dictionary of arguments to be passed to Bokeh, and
+    resources files may optionally be omitted from loading by passing
+    resources="none".
+    '''
+    return _connect_session_or_document(model=None, session_id=None, url=url, relative_urls=relative_urls,
+                                        resources=resources, arguments=arguments)
+
+def server_session(model, session_id, url="default", relative_urls=False, resources="default", arguments=None):
+    ''' Works similarly to ``autoload_server`` except an existing session id and
+    model must be provided. Also accepts an optional dictionary of arguments to
+    be passed to Bokeh, and resources files may optionally be omitted from
+    loading by passing resources="none".
+    '''
+    return _connect_session_or_document(model, session_id=session_id, url=url, relative_urls=relative_urls,
+                                        resources=resources, arguments=arguments)
 
 def _script_for_render_items(docs_json, render_items, app_path=None, absolute_url=None):
     js = DOC_JS.render(
